@@ -21,16 +21,41 @@
 
 #include "wine/debug.h"
 
+#include <limits.h>
 #define COBJMACROS
 #include "d2d1.h"
+#ifdef D2D1_INIT_GUID
+#include "initguid.h"
+#endif
+#include "dwrite.h"
+
+struct d2d_clip_stack
+{
+    D2D1_RECT_F *stack;
+    unsigned int size;
+    unsigned int count;
+};
 
 struct d2d_d3d_render_target
 {
     ID2D1RenderTarget ID2D1RenderTarget_iface;
+    IDWriteTextRenderer IDWriteTextRenderer_iface;
     LONG refcount;
 
+    ID3D10Device *device;
+    ID3D10RenderTargetView *view;
+    ID3D10StateBlock *stateblock;
+
+    ID3D10InputLayout *clear_il;
+    unsigned int clear_vb_stride;
+    ID3D10Buffer *clear_vb;
+    ID3D10VertexShader *clear_vs;
+    ID3D10PixelShader *clear_ps;
+    ID3D10RasterizerState *clear_rs;
+
     D2D1_SIZE_U pixel_size;
     D2D1_MATRIX_3X2_F transform;
+    struct d2d_clip_stack clip_stack;
     float dpi_x;
     float dpi_y;
 };
@@ -38,6 +63,24 @@ struct d2d_d3d_render_target
 HRESULT d2d_d3d_render_target_init(struct d2d_d3d_render_target *render_target, ID2D1Factory *factory,
         IDXGISurface *surface, const D2D1_RENDER_TARGET_PROPERTIES *desc) DECLSPEC_HIDDEN;
 
+struct d2d_wic_render_target
+{
+    ID2D1RenderTarget ID2D1RenderTarget_iface;
+    LONG refcount;
+
+    IDXGISurface *dxgi_surface;
+    ID2D1RenderTarget *dxgi_target;
+    ID3D10Texture2D *readback_texture;
+    IWICBitmap *bitmap;
+
+    unsigned int width;
+    unsigned int height;
+    unsigned int bpp;
+};
+
+HRESULT d2d_wic_render_target_init(struct d2d_wic_render_target *render_target, ID2D1Factory *factory,
+        IWICBitmap *bitmap, const D2D1_RENDER_TARGET_PROPERTIES *desc) DECLSPEC_HIDDEN;
+
 struct d2d_gradient
 {
     ID2D1GradientStopCollection ID2D1GradientStopCollection_iface;
@@ -69,4 +112,21 @@ struct d2d_stroke_style
 void d2d_stroke_style_init(struct d2d_stroke_style *style, ID2D1Factory *factory,
         const D2D1_STROKE_STYLE_PROPERTIES *desc, const float *dashes, UINT32 dash_count) DECLSPEC_HIDDEN;
 
+struct d2d_mesh
+{
+    ID2D1Mesh ID2D1Mesh_iface;
+    LONG refcount;
+};
+
+void d2d_mesh_init(struct d2d_mesh *mesh) DECLSPEC_HIDDEN;
+
+struct d2d_bitmap
+{
+    ID2D1Bitmap ID2D1Bitmap_iface;
+    LONG refcount;
+};
+
+void d2d_bitmap_init(struct d2d_bitmap *bitmap, D2D1_SIZE_U size, const void *src_data,
+        UINT32 pitch, const D2D1_BITMAP_PROPERTIES *desc) DECLSPEC_HIDDEN;
+
 #endif /* __WINE_D2D1_PRIVATE_H */
