@@ -478,7 +478,7 @@ static void test_GetLocaleName(void)
     static const WCHAR strW[] = {'s','t','r','i','n','g',0};
     static const WCHAR ruW[] = {'r','u',0};
     IDWriteTextLayout *layout;
-    IDWriteTextFormat *format;
+    IDWriteTextFormat *format, *format2;
     WCHAR buff[10];
     UINT32 len;
     HRESULT hr;
@@ -489,13 +489,17 @@ static void test_GetLocaleName(void)
 
     hr = IDWriteFactory_CreateGdiCompatibleTextLayout(factory, strW, 0, format, 100.0, 100.0, 1.0, NULL, FALSE, &layout);
     ok(hr == S_OK, "got 0x%08x\n", hr);
-    len = IDWriteTextLayout_GetLocaleNameLength(layout);
+
+    hr = IDWriteTextLayout_QueryInterface(layout, &IID_IDWriteTextFormat, (void**)&format2);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    len = IDWriteTextFormat_GetLocaleNameLength(format2);
     ok(len == 2, "got %u\n", len);
     len = IDWriteTextFormat_GetLocaleNameLength(format);
     ok(len == 2, "got %u\n", len);
-    hr = IDWriteTextLayout_GetLocaleName(layout, buff, len);
+    hr = IDWriteTextFormat_GetLocaleName(format2, buff, len);
     ok(hr == E_NOT_SUFFICIENT_BUFFER, "got 0x%08x\n", hr);
-    hr = IDWriteTextLayout_GetLocaleName(layout, buff, len+1);
+    hr = IDWriteTextFormat_GetLocaleName(format2, buff, len+1);
     ok(hr == S_OK, "got 0x%08x\n", hr);
     ok(!lstrcmpW(buff, ruW), "got %s\n", wine_dbgstr_w(buff));
     hr = IDWriteTextFormat_GetLocaleName(format, buff, len);
@@ -506,6 +510,7 @@ static void test_GetLocaleName(void)
 
     IDWriteTextLayout_Release(layout);
     IDWriteTextFormat_Release(format);
+    IDWriteTextFormat_Release(format2);
 }
 
 static void test_CreateEllipsisTrimmingSign(void)
@@ -545,6 +550,7 @@ static void test_fontweight(void)
     IDWriteTextLayout *layout;
     DWRITE_FONT_WEIGHT weight;
     DWRITE_TEXT_RANGE range;
+    FLOAT size;
     HRESULT hr;
 
     hr = IDWriteFactory_CreateTextFormat(factory, tahomaW, NULL, DWRITE_FONT_WEIGHT_BOLD, DWRITE_FONT_STYLE_NORMAL,
@@ -571,11 +577,53 @@ static void test_fontweight(void)
 
     range.length = 0;
     weight = DWRITE_FONT_WEIGHT_BOLD;
-    hr = layout->lpVtbl->IDWriteTextLayout_GetFontWeight(layout, 0, &weight, &range);
+    hr = IDWriteTextLayout_GetFontWeight(layout, 0, &weight, &range);
     ok(hr == S_OK, "got 0x%08x\n", hr);
     ok(weight == DWRITE_FONT_WEIGHT_NORMAL, "got %d\n", weight);
     ok(range.length == 6, "got %d\n", range.length);
 
+    size = IDWriteTextLayout_GetMaxWidth(layout);
+    ok(size == 100.0, "got %.2f\n", size);
+
+    hr = IDWriteTextLayout_SetMaxWidth(layout, 0.0);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    size = IDWriteTextLayout_GetMaxWidth(layout);
+    ok(size == 0.0, "got %.2f\n", size);
+
+    hr = IDWriteTextLayout_SetMaxWidth(layout, -1.0);
+    ok(hr == E_INVALIDARG, "got 0x%08x\n", hr);
+
+    size = IDWriteTextLayout_GetMaxWidth(layout);
+    ok(size == 0.0, "got %.2f\n", size);
+
+    hr = IDWriteTextLayout_SetMaxWidth(layout, 100.0);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    size = IDWriteTextLayout_GetMaxWidth(layout);
+    ok(size == 100.0, "got %.2f\n", size);
+
+    size = IDWriteTextLayout_GetMaxHeight(layout);
+    ok(size == 100.0, "got %.2f\n", size);
+
+    hr = IDWriteTextLayout_SetMaxHeight(layout, 0.0);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    size = IDWriteTextLayout_GetMaxHeight(layout);
+    ok(size == 0.0, "got %.2f\n", size);
+
+    hr = IDWriteTextLayout_SetMaxHeight(layout, -1.0);
+    ok(hr == E_INVALIDARG, "got 0x%08x\n", hr);
+
+    size = IDWriteTextLayout_GetMaxHeight(layout);
+    ok(size == 0.0, "got %.2f\n", size);
+
+    hr = IDWriteTextLayout_SetMaxHeight(layout, 100.0);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    size = IDWriteTextLayout_GetMaxHeight(layout);
+    ok(size == 100.0, "got %.2f\n", size);
+
     IDWriteTextLayout_Release(layout);
     IDWriteTextFormat_Release(fmt2);
     IDWriteTextFormat_Release(format);
@@ -750,6 +798,51 @@ todo_wine
     IDWriteTextLayout_Release(layout);
 }
 
+static void test_typography(void)
+{
+    DWRITE_FONT_FEATURE feature;
+    IDWriteTypography *typography;
+    UINT32 count;
+    HRESULT hr;
+
+    hr = IDWriteFactory_CreateTypography(factory, &typography);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    feature.nameTag = DWRITE_FONT_FEATURE_TAG_KERNING;
+    feature.parameter = 1;
+    hr = IDWriteTypography_AddFontFeature(typography, feature);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    count = IDWriteTypography_GetFontFeatureCount(typography);
+    ok(count == 1, "got %u\n", count);
+
+    /* duplicated features work just fine */
+    feature.nameTag = DWRITE_FONT_FEATURE_TAG_KERNING;
+    feature.parameter = 0;
+    hr = IDWriteTypography_AddFontFeature(typography, feature);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    count = IDWriteTypography_GetFontFeatureCount(typography);
+    ok(count == 2, "got %u\n", count);
+
+    memset(&feature, 0, sizeof(feature));
+    hr = IDWriteTypography_GetFontFeature(typography, 0, &feature);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+    ok(feature.nameTag == DWRITE_FONT_FEATURE_TAG_KERNING, "got tag %x\n", feature.nameTag);
+    ok(feature.parameter == 1, "got %u\n", feature.parameter);
+
+    memset(&feature, 0, sizeof(feature));
+    hr = IDWriteTypography_GetFontFeature(typography, 1, &feature);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+    ok(feature.nameTag == DWRITE_FONT_FEATURE_TAG_KERNING, "got tag %x\n", feature.nameTag);
+    ok(feature.parameter == 0, "got %u\n", feature.parameter);
+
+    hr = IDWriteTypography_GetFontFeature(typography, 2, &feature);
+    ok(hr == E_INVALIDARG, "got 0x%08x\n", hr);
+
+    IDWriteTypography_Release(typography);
+}
+
 START_TEST(layout)
 {
     HRESULT hr;
@@ -773,6 +866,7 @@ START_TEST(layout)
     test_fontweight();
     test_SetInlineObject();
     test_draw_sequence();
+    test_typography();
 
     IDWriteFactory_Release(factory);
 }
