@@ -29,6 +29,7 @@
 #include "ole2.h"
 #include "activscp.h"
 #include "activdbg.h"
+#include "shlwapi.h"
 
 #include "wine/debug.h"
 
@@ -701,7 +702,15 @@ static ScriptHost *create_script_host(HTMLInnerWindow *window, const GUID *guid)
     return ret;
 }
 
-static void parse_text(ScriptHost *script_host, LPCWSTR text)
+static void set_script_elem_readystate(HTMLScriptElement *script_elem, READYSTATE readystate)
+{
+    script_elem->readystate = readystate;
+
+    if(readystate != READYSTATE_INTERACTIVE)
+        fire_event(script_elem->element.node.doc, EVENTID_READYSTATECHANGE, FALSE, script_elem->element.node.nsnode, NULL, NULL);
+}
+
+static void parse_elem_text(ScriptHost *script_host, HTMLScriptElement *script_elem, LPCWSTR text)
 {
     EXCEPINFO excepinfo;
     VARIANT var;
@@ -709,6 +718,8 @@ static void parse_text(ScriptHost *script_host, LPCWSTR text)
 
     TRACE("%s\n", debugstr_w(text));
 
+    set_script_elem_readystate(script_elem, READYSTATE_INTERACTIVE);
+
     VariantInit(&var);
     memset(&excepinfo, 0, sizeof(excepinfo));
     TRACE(">>>\n");
@@ -722,27 +733,233 @@ static void parse_text(ScriptHost *script_host, LPCWSTR text)
 
 }
 
-static void parse_extern_script(ScriptHost *script_host, LPCWSTR src)
+typedef struct {
+    BSCallback bsc;
+
+    HTMLScriptElement *script_elem;
+    DWORD scheme;
+
+    DWORD size;
+    char *buf;
+    HRESULT hres;
+} ScriptBSC;
+
+static inline ScriptBSC *impl_from_BSCallback(BSCallback *iface)
 {
+    return CONTAINING_RECORD(iface, ScriptBSC, bsc);
+}
+
+static void ScriptBSC_destroy(BSCallback *bsc)
+{
+    ScriptBSC *This = impl_from_BSCallback(bsc);
+
+    if(This->script_elem) {
+        IHTMLScriptElement_Release(&This->script_elem->IHTMLScriptElement_iface);
+        This->script_elem = NULL;
+    }
+
+    heap_free(This->buf);
+    heap_free(This);
+}
+
+static HRESULT ScriptBSC_init_bindinfo(BSCallback *bsc)
+{
+    return S_OK;
+}
+
+static HRESULT ScriptBSC_start_binding(BSCallback *bsc)
+{
+    ScriptBSC *This = impl_from_BSCallback(bsc);
+
+    /* FIXME: We should find a better to decide if 'loading' state is supposed to be used by the protocol. */
+    if(This->scheme == URL_SCHEME_HTTPS || This->scheme == URL_SCHEME_HTTP)
+        set_script_elem_readystate(This->script_elem, READYSTATE_LOADING);
+
+    return S_OK;
+}
+
+static HRESULT ScriptBSC_stop_binding(BSCallback *bsc, HRESULT result)
+{
+    ScriptBSC *This = impl_from_BSCallback(bsc);
+
+    This->hres = result;
+
+    if(SUCCEEDED(result)) {
+        if(This->script_elem->readystate == READYSTATE_LOADING)
+            set_script_elem_readystate(This->script_elem, READYSTATE_LOADED);
+    }else {
+        FIXME("binding failed %08x\n", result);
+        heap_free(This->buf);
+        This->buf = NULL;
+        This->size = 0;
+    }
+
+    IHTMLScriptElement_Release(&This->script_elem->IHTMLScriptElement_iface);
+    This->script_elem = NULL;
+    return S_OK;
+}
+
+static HRESULT ScriptBSC_read_data(BSCallback *bsc, IStream *stream)
+{
+    ScriptBSC *This = impl_from_BSCallback(bsc);
+    DWORD readed;
+    HRESULT hres;
+
+    if(!This->buf) {
+        This->buf = heap_alloc(128);
+        if(!This->buf)
+            return E_OUTOFMEMORY;
+        This->size = 128;
+    }
+
+    do {
+        if(This->bsc.readed >= This->size) {
+	  void *new_buf;
+	  new_buf = heap_realloc(This->buf, This->size << 1);
+	  if(!new_buf)
+	    return E_OUTOFMEMORY;
+            This->size <<= 1;
+            This->buf = new_buf;
+        }
+
+        hres = read_stream(&This->bsc, stream, This->buf+This->bsc.readed, This->size-This->bsc.readed, &readed);
+    }while(hres == S_OK);
+
+    return S_OK;
+}
+
+static HRESULT ScriptBSC_on_progress(BSCallback *bsc, ULONG status_code, LPCWSTR status_text)
+{
+    return S_OK;
+}
+
+static HRESULT ScriptBSC_on_response(BSCallback *bsc, DWORD response_code,
+        LPCWSTR response_headers)
+{
+    return S_OK;
+}
+
+static HRESULT ScriptBSC_beginning_transaction(BSCallback *bsc, WCHAR **additional_headers)
+{
+    return S_FALSE;
+}
+
+static const BSCallbackVtbl ScriptBSCVtbl = {
+    ScriptBSC_destroy,
+    ScriptBSC_init_bindinfo,
+    ScriptBSC_start_binding,
+    ScriptBSC_stop_binding,
+    ScriptBSC_read_data,
+    ScriptBSC_on_progress,
+    ScriptBSC_on_response,
+    ScriptBSC_beginning_transaction
+};
+
+
+static HRESULT bind_script_to_text(HTMLInnerWindow *window, IUri *uri, HTMLScriptElement *script_elem, WCHAR **ret)
+{
+    ScriptBSC *bsc;
     IMoniker *mon;
     WCHAR *text;
     HRESULT hres;
 
+    hres = CreateURLMonikerEx2(NULL, uri, &mon, URL_MK_UNIFORM);
+    if(FAILED(hres))
+        return hres;
+
+    bsc = heap_alloc_zero(sizeof(*bsc));
+    if(!bsc) {
+        IMoniker_Release(mon);
+        return E_OUTOFMEMORY;
+    }
+
+    init_bscallback(&bsc->bsc, &ScriptBSCVtbl, mon, 0);
+    IMoniker_Release(mon);
+    bsc->hres = E_FAIL;
+
+    hres = IUri_GetScheme(uri, &bsc->scheme);
+    if(FAILED(hres))
+        bsc->scheme = URL_SCHEME_UNKNOWN;
+
+    IHTMLScriptElement_AddRef(&script_elem->IHTMLScriptElement_iface);
+    bsc->script_elem = script_elem;
+
+    hres = start_binding(window, &bsc->bsc, NULL);
+    if(SUCCEEDED(hres))
+        hres = bsc->hres;
+    if(FAILED(hres)) {
+        IBindStatusCallback_Release(&bsc->bsc.IBindStatusCallback_iface);
+        return hres;
+    }
+
+    if(!bsc->bsc.readed) {
+        *ret = NULL;
+        return S_OK;
+    }
+
+    switch(bsc->bsc.bom) {
+    case BOM_UTF16:
+        if(bsc->bsc.readed % sizeof(WCHAR)) {
+            FIXME("The buffer is not a valid utf16 string\n");
+            hres = E_FAIL;
+            break;
+        }
+
+        text = heap_alloc(bsc->bsc.readed+sizeof(WCHAR));
+        if(!text) {
+            hres = E_OUTOFMEMORY;
+            break;
+        }
+
+        memcpy(text, bsc->buf, bsc->bsc.readed);
+        text[bsc->bsc.readed/sizeof(WCHAR)] = 0;
+        break;
+
+    case BOM_UTF8:
+    default: {
+        DWORD len;
+
+        len = MultiByteToWideChar(CP_UTF8, 0, bsc->buf, bsc->bsc.readed, NULL, 0);
+        text = heap_alloc((len+1)*sizeof(WCHAR));
+        if(!text) {
+            hres = E_OUTOFMEMORY;
+            break;
+        }
+
+        MultiByteToWideChar(CP_UTF8, 0, bsc->buf, bsc->bsc.readed, text, len);
+        text[len] = 0;
+    }
+    }
+
+    IBindStatusCallback_Release(&bsc->bsc.IBindStatusCallback_iface);
+    if(FAILED(hres))
+        return hres;
+
+    *ret = text;
+    return S_OK;
+}
+
+static void parse_extern_script(ScriptHost *script_host, HTMLScriptElement *script_elem, LPCWSTR src)
+{
+    WCHAR *text;
+    IUri *uri;
+    HRESULT hres;
+
     static const WCHAR wine_schemaW[] = {'w','i','n','e',':'};
 
     if(strlenW(src) > sizeof(wine_schemaW)/sizeof(WCHAR) && !memcmp(src, wine_schemaW, sizeof(wine_schemaW)))
         src += sizeof(wine_schemaW)/sizeof(WCHAR);
 
-    hres = CreateURLMoniker(NULL, src, &mon);
+    hres = create_uri(src, 0, &uri);
     if(FAILED(hres))
         return;
 
-    hres = bind_mon_to_wstr(script_host->window, mon, &text);
-    IMoniker_Release(mon);
-    if(FAILED(hres))
+    hres = bind_script_to_text(script_host->window, uri, script_elem, &text);
+    IUri_Release(uri);
+    if(FAILED(hres) || !text)
         return;
 
-    parse_text(script_host, text);
+    parse_elem_text(script_host, script_elem, text);
 
     heap_free(text);
 }
@@ -760,7 +977,7 @@ static void parse_inline_script(ScriptHost *script_host, HTMLScriptElement *scri
     if(NS_FAILED(nsres)) {
         ERR("GetText failed: %08x\n", nsres);
     }else if(*text) {
-        parse_text(script_host, text);
+        parse_elem_text(script_host, script_elem, text);
     }
 
     nsAString_Finish(&text_str);
@@ -796,12 +1013,14 @@ static void parse_script_elem(ScriptHost *script_host, HTMLScriptElement *script
         ERR("GetSrc failed: %08x\n", nsres);
     }else if(*src) {
         script_elem->parsed = TRUE;
-        parse_extern_script(script_host, src);
+        parse_extern_script(script_host, script_elem, src);
     }else {
         parse_inline_script(script_host, script_elem);
     }
 
     nsAString_Finish(&src_str);
+
+    set_script_elem_readystate(script_elem, READYSTATE_COMPLETE);
 }
 
 static GUID get_default_script_guid(HTMLInnerWindow *window)
@@ -849,6 +1068,7 @@ static BOOL get_guid_from_language(LPCWSTR type, GUID *guid)
 
 static BOOL get_script_guid(HTMLInnerWindow *window, nsIDOMHTMLScriptElement *nsscript, GUID *guid)
 {
+    nsIDOMHTMLElement *nselem;
     const PRUnichar *language;
     nsAString val_str;
     BOOL ret = FALSE;
@@ -872,7 +1092,11 @@ static BOOL get_script_guid(HTMLInnerWindow *window, nsIDOMHTMLScriptElement *ns
         ERR("GetType failed: %08x\n", nsres);
     }
 
-    nsres = get_elem_attr_value((nsIDOMHTMLElement*)nsscript, languageW, &val_str, &language);
+    nsres = nsIDOMHTMLScriptElement_QueryInterface(nsscript, &IID_nsIDOMHTMLElement, (void**)&nselem);
+    assert(nsres == NS_OK);
+
+    nsres = get_elem_attr_value(nselem, languageW, &val_str, &language);
+    nsIDOMHTMLElement_Release(nselem);
     if(NS_SUCCEEDED(nsres)) {
         if(*language) {
             ret = get_guid_from_language(language, guid);
