@@ -57,6 +57,7 @@ typedef enum
     WIN2K8R2,/* Windows 2008 R2 */
     WIN7,    /* Windows 7 */
     WIN8,    /* Windows 8 */
+    WIN81,   /* Windows 8.1 */
     NB_WINDOWS_VERSIONS
 } WINDOWS_VERSION;
 
@@ -177,6 +178,12 @@ static const RTL_OSVERSIONINFOEXW VersionData[NB_WINDOWS_VERSIONS] =
         {' ',0},
         0, 0, VER_SUITE_SINGLEUSERTS, VER_NT_WORKSTATION, 0
     },
+    /* WIN81 */
+    {
+        sizeof(RTL_OSVERSIONINFOEXW), 6, 3, 0x2580, VER_PLATFORM_WIN32_NT,
+        {' ',0},
+        0, 0, VER_SUITE_SINGLEUSERTS, VER_NT_WORKSTATION, 0
+    },
 
 };
 
@@ -199,6 +206,7 @@ static const char * const WinVersionNames[NB_WINDOWS_VERSIONS] =
     "win2008r2,win2k8r2",         /* WIN2K8R2 */
     "win7",                       /* WIN7 */
     "win8",                       /* WIN8 */
+    "win81",                      /* WIN81 */
 };
 
 
