@@ -366,6 +366,26 @@ CONFIGRET WINAPI CM_Locate_DevNode_ExW(PDEVINST pdnDevInst, DEVINSTID_W pDeviceI
     return CR_FAILURE;
 }
 
+/***********************************************************************
+ *      CM_Get_Device_Interface_List_SizeA (SETUPAPI.@)
+ */
+CONFIGRET WINAPI CM_Get_Device_Interface_List_SizeA(PULONG len, LPGUID class, DEVINSTID_A id,
+                                                    ULONG flags)
+{
+    FIXME("%p %p %s 0x%08x: stub\n", len, class, debugstr_a(id), flags);
+    return CR_FAILURE;
+}
+
+/***********************************************************************
+ *      CM_Get_Device_Interface_List_SizeW (SETUPAPI.@)
+ */
+CONFIGRET WINAPI CM_Get_Device_Interface_List_SizeW(PULONG len, LPGUID class, DEVINSTID_W id,
+                                                    ULONG flags)
+{
+    FIXME("%p %p %s 0x%08x: stub\n", len, class, debugstr_w(id), flags);
+    return CR_FAILURE;
+}
+
 /***********************************************************************
  *      CM_Get_Device_Interface_List_Size_ExA (SETUPAPI.@)
  */
